@@ -78,7 +78,7 @@ func init() {
 		"Path to local file or directory.")
 	pushArtifactCmd.Flags().Var(&pushArtifactArgs.creds, pushArtifactArgs.creds.Type(), pushArtifactArgs.creds.Description())
 	pushArtifactCmd.Flags().StringArrayVarP(&pushArtifactArgs.tags, "tag", "t", nil,
-		"TagArtifact of the artifact.")
+		"Tag of the artifact.")
 	pushArtifactCmd.Flags().StringArrayVarP(&pushArtifactArgs.annotations, "annotation", "a", nil,
 		"Annotation in the format '<key>=<value>'.")
 	pushArtifactCmd.Flags().StringVar(&pushArtifactArgs.contentType, "content-type", "generic",
@@ -131,7 +131,7 @@ func pushArtifactCmdRun(cmd *cobra.Command, args []string) error {
 	spin := StartSpinner("pushing artifact")
 	defer spin.Stop()
 
-	opts := oci.Options(ctx, pushArtifactArgs.creds.String())
+	opts := oci.Options(ctx, pushArtifactArgs.creds.String(), rootArgs.registryInsecure)
 	ociURL := fmt.Sprintf("%s:%s", args[0], pushArtifactArgs.tags[0])
 	digestURL, err := oci.PushArtifact(ociURL,
 		pushArtifactArgs.path,
